@@ -4,7 +4,7 @@ Structure, Field, StructureReference, ClassReference, TypedField
 """
 import enum
 from copy import deepcopy
-from collections import OrderedDict, deque
+from collections import OrderedDict, deque, defaultdict
 from inspect import Signature, Parameter, signature
 import sys
 import typing
@@ -12,13 +12,15 @@ import hashlib
 
 from typing import get_type_hints, Iterable
 
-from typedpy.commons import wrap_val
+from typedpy.commons import wrap_val, _is_sunder, _is_dunder
 
 REQUIRED_FIELDS = "_required"
 DEFAULTS = "_defaults"
 ADDITIONAL_PROPERTIES = "_additionalProperties"
 IS_IMMUTABLE = "_immutable"
 OPTIONAL_FIELDS = "_optional"
+MUST_BE_UNIQUE = "_must_be_unique"
+MAX_NUMBER_OF_INSTANCES_TO_VERIFY_UNIQUENESS = 100000
 
 py_version = sys.version_info[0:2]
 python_ver_36 = py_version == (3, 6)
@@ -106,9 +108,15 @@ def get_base_info(bases):
     """
     bases_params = OrderedDict()
     bases_required = []
-    base_structures = [
-        base for base in bases if issubclass(base, Structure) and base is not Structure
-    ]
+    base_structures = (
+        [
+            base
+            for base in bases
+            if issubclass(base, Structure) and base is not Structure
+        ]
+        if "Structure" in globals()
+        else []
+    )
     for base in base_structures:
         for k, param in getattr(base, "__signature__").parameters.items():
             if k not in bases_params:
@@ -122,9 +130,45 @@ def get_base_info(bases):
     return bases_params, bases_required
 
 
+def _check_for_final_violations(classes):
+    def is_sub_class(c, base):
+        return issubclass(c, base) and c != base
+
+    current_class, *inherited_class = classes
+    for i, c in enumerate(inherited_class):
+        if "FinalStructure" in globals() and isinstance(c, StructMeta):
+            if "FinalStructure" in globals() and is_sub_class(c, FinalStructure):
+                raise TypeError(
+                    "Tried to extend {}, which is a FinalStructure. This is forbidden".format(
+                        c.__name__
+                    )
+                )
+            if "ImmutableStructure" in globals() and is_sub_class(
+                c, ImmutableStructure
+            ):
+                raise TypeError(
+                    "Tried to extend {}, which is an ImmutableStructure. This is forbidden".format(
+                        c.__name__
+                    )
+                )
+
+        if "_FieldMeta" in globals() and isinstance(c, _FieldMeta):
+            if "ImmutableField" in globals() and is_sub_class(c, ImmutableField):
+                raise TypeError(
+                    "Tried to extend {}, which is an ImmutableField. This is forbidden".format(
+                        c.__name__
+                    )
+                )
+
+
 class _FieldMeta(type):
     _registry = {}
 
+    def __new__(cls, name, bases, cls_dict):
+        clsobj = super().__new__(cls, name, bases, dict(cls_dict))
+        _check_for_final_violations(clsobj.mro())
+        return clsobj
+
     def __getitem__(cls, val):
         if isinstance(val, Field):
             return val
@@ -158,15 +202,99 @@ class _FieldMeta(type):
             return class_as_field()
 
 
-class Field(metaclass=_FieldMeta):
+class UniqueMixin:
+    def defined_as_unique(self):
+        return getattr(self, MUST_BE_UNIQUE, False)
+
+    def __manage_uniqueness__(self):
+        myclass = self.__class__
+        if (
+            getattr(myclass, MUST_BE_UNIQUE, False)
+            and len(getattr(myclass, "_ALL_INSTANCES", set()))
+            < MAX_NUMBER_OF_INSTANCES_TO_VERIFY_UNIQUENESS
+        ):
+            hash_of_instance = self.__hash__()
+            if hash_of_instance in getattr(myclass, "_ALL_INSTANCES", set()):
+                classname = self.__class__.__name__
+                raise ValueError(
+                    "Instance copy in {}, which is defined as unique. Instance is {}".format(
+                        classname, self
+                    )
+                )
+            getattr(myclass, "_ALL_INSTANCES", set()).add(hash_of_instance)
+
+    def __manage_uniqueness_for_field__(self, instance, value):
+        if not getattr(instance, "_instantiated", False):
+            return
+        field_name = getattr(self, "_name")
+        structure_class_name = instance.__class__.__name__
+        all_instances_by_struct_name = getattr(
+            self, "_ALL_INSTANCES", defaultdict(dict)
+        )
+        instance_by_value_for_current_struct = all_instances_by_struct_name[
+            structure_class_name
+        ]
+        if (
+            getattr(self, MUST_BE_UNIQUE, False)
+            and len(instance_by_value_for_current_struct)
+            < MAX_NUMBER_OF_INSTANCES_TO_VERIFY_UNIQUENESS
+        ):
+            hash_of_field_val = value.__hash__()
+            if (
+                instance_by_value_for_current_struct.get(hash_of_field_val, instance)
+                != instance
+            ):
+                raise ValueError(
+                    "Instance copy of field {} in {}, which is defined as unique. Instance is {}".format(
+                        field_name, structure_class_name, wrap_val(value)
+                    )
+                )
+            if hash_of_field_val not in instance_by_value_for_current_struct:
+                instance_by_value_for_current_struct[hash_of_field_val] = instance
+
+
+class Field(UniqueMixin, metaclass=_FieldMeta):
     """
     Base class for a field(i.e. property) in a structure.
     Should not be used directly by developers.
+    Arguments:
+        immutable: optional
+            Marks the field as immutable. Typically the developer does not need to use it,
+            as there is a high level API for making a field immutable
+
+        is_unique: optional
+            Marks a field as unique within this its Structure.
+            as there is a high level API for making a field immutable
+
+            .. code-block:: python
+
+                    class SSID(String): pass
+
+                    class Person(Structure):
+                            ssid: SSID(is_unique=True)
+                            name: String
+
+                    Person(ssid="1234", name="john")
+                    # the next line will raise an exception "Instance copy of field ssid in Person"
+                    Person(ssid="1234", name="jeff")
+
+            Alternatively, you can use the "@unique" decorator on the class definition of the Field.
+            Refer to "Uniqueness" section for more detail.
+
+        default: optional
+            default value in case no value was assigned. Setting it makes
+            the field implicitly optional.
+            Default values are validated based on the field definition like any other value
+            assignment.
     """
 
-    def __init__(self, name=None, immutable=None, default=None):
+    def __init__(self, name=None, immutable=None, is_unique=None, default=None):
         self._name = name
         self._default = default
+        if is_unique in [True, False]:
+            setattr(self, MUST_BE_UNIQUE, is_unique)
+            if is_unique:
+                self._ALL_INSTANCES = defaultdict(dict)
         if immutable is not None:
             self._immutable = immutable
         if default:
@@ -216,7 +344,9 @@ class Field(metaclass=_FieldMeta):
                     )
                 )
         else:
+            self.__manage_uniqueness_for_field__(instance, value)
             instance.__dict__[self._name] = value
+            instance.__manage__uniqueness_of_all_fields__()
         if getattr(instance, "_instantiated", False) and not getattr(
             instance, "_skip_validation", False
         ):
@@ -351,6 +481,7 @@ class StructMeta(type):
 
         cls_dict.pop(DEFAULTS, None)
         clsobj = super().__new__(cls, name, bases, dict(cls_dict))
+        _check_for_final_violations(clsobj.mro())
 
         clsobj._fields = fields
         default_required = (
@@ -500,7 +631,7 @@ def add_annotations_to_class_dict(cls_dict):
     cls_dict[DEFAULTS] = defaults
 
 
-class Structure(metaclass=StructMeta):
+class Structure(UniqueMixin, metaclass=StructMeta):
     """
     The base class to support strictly defined structures. When creating a new instance of
     a Structure, fields must be provided by name.
@@ -555,6 +686,8 @@ class Structure(metaclass=StructMeta):
                 # this raises an exception:
                 Foo(id = 1, a = 2)
 
+    Decorating it with @unique ensures that no all instances of this structure will be unique. It
+    will raise an exception otherwise (see "Uniqueness" section).
     """
 
     _fields = []
@@ -570,6 +703,14 @@ class Structure(metaclass=StructMeta):
 
         self.__validate__()
         self._instantiated = True
+        self.__manage_uniqueness__()
+        self.__manage__uniqueness_of_all_fields__()
+
+    def __manage__uniqueness_of_all_fields__(self):
+        fields_by_name = _get_all_fields_by_name(self.__class__)
+        for name, field in fields_by_name.items():
+            if field.defined_as_unique():
+                field.__manage_uniqueness_for_field__(self, getattr(self, name, None))
 
     def __setattr__(self, key, value):
         if getattr(self, IS_IMMUTABLE, False):
@@ -578,6 +719,13 @@ class Structure(metaclass=StructMeta):
             value = deepcopy(value)
         super().__setattr__(key, value)
 
+        if (
+            getattr(self, "_instantiated", False)
+            and not _is_dunder(key)
+            and not _is_sunder(key)
+        ):
+            self.__manage_uniqueness__()
+
     def __getstate__(self):
         fields_by_name = _get_all_fields_by_name(self.__class__)
         return {
@@ -697,6 +845,20 @@ class Structure(metaclass=StructMeta):
         )
 
 
+class FinalStructure(Structure):
+    pass
+
+
+def unique(cls):
+    if issubclass(cls, Structure):
+        setattr(cls, MUST_BE_UNIQUE, True)
+        cls._ALL_INSTANCES = set()
+    elif issubclass(cls, Field):
+        setattr(cls, MUST_BE_UNIQUE, True)
+        cls._ALL_INSTANCES = defaultdict(dict)
+    return cls
+
+
 class ImmutableStructure(Structure):
     """
     A base class for a structure in which non of the fields can be updated post-creation
@@ -719,6 +881,8 @@ class ImmutableStructure(Structure):
         b.z.clear()
         b.m.pop('a')
 
+    ImmutableStructure class (as the class B in the example above) are not allowed to be extended.
+    This is to ensure any instance of ImmutableStructure is indeed immutable.
     """
 
     _immutable = True
@@ -750,6 +914,7 @@ class TypedField(Field):
 class NoneField(TypedField):
     """
     A field that maps to a single allowable value: None.
+    By default, fields cannot be assigned None (i.e. "Null Safety"). NoneField allows to do so.
     This is useful to define optional fields or optional values such as:
 
     .. code-block:: python
@@ -824,3 +989,24 @@ class ClassReference(TypedField):
 
     def __str__(self):
         return "<ClassReference: {}>".format(self._ty.__name__)
+
+
+class ImmutableField(Field):
+    """
+    A mixin that makes a field class immutable.
+    For Example:
+
+     .. code-block:: python
+
+         class MyFieldType(Field): .....
+
+         class MyImmutableFieldType(ImmutableField, MyFieldType): pass
+
+         # that's all you have to do to make MyImmutableFieldType immutable.
+
+    ImmutableField class (as the class MyImmutableFieldType in the example above) are
+    not allowed to be extended. This is to ensure any instance of
+    ImmutableField is indeed immutable.
+    """
+
+    _immutable = True
